#include <kdialog.h>
#include <klocale.h>

/********************************************************************************
** Form generated from reading UI file 'kjserrordlgbase.ui'
**
** Created by: Qt User Interface Compiler version 4.8.7
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KJSERRORDLGBASE_H
#define UI_KJSERRORDLGBASE_H

#include <QtCore/QVariant>
#include <QtGui/QAction>
#include <QtGui/QApplication>
#include <QtGui/QButtonGroup>
#include <QtGui/QDialog>
#include <QtGui/QFrame>
#include <QtGui/QGridLayout>
#include <QtGui/QHeaderView>
#include <QtGui/QLabel>
#include <QtGui/QPushButton>
#include <QtGui/QSpacerItem>
#include <QtGui/QTextBrowser>
#include <kdialog.h>

QT_BEGIN_NAMESPACE

class Ui_KJSErrorDlgBase
{
public:
    QGridLayout *gridLayout;
    QPushButton *_close;
    QPushButton *_clear;
    QFrame *line1;
    QSpacerItem *spacerItem;
    QTextBrowser *_errorText;
    QLabel *_url;

    void setupUi(QDialog *KJSErrorDlgBase)
    {
        if (KJSErrorDlgBase->objectName().isEmpty())
            KJSErrorDlgBase->setObjectName(QString::fromUtf8("KJSErrorDlgBase"));
        KJSErrorDlgBase->resize(600, 484);
        gridLayout = new QGridLayout(KJSErrorDlgBase);
#ifndef UI_Q_OS_MAC
        gridLayout->setSpacing(6);
#endif
        gridLayout->setContentsMargins(11, 11, 11, 11);
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        _close = new QPushButton(KJSErrorDlgBase);
        _close->setObjectName(QString::fromUtf8("_close"));

        gridLayout->addWidget(_close, 4, 2, 1, 1);

        _clear = new QPushButton(KJSErrorDlgBase);
        _clear->setObjectName(QString::fromUtf8("_clear"));

        gridLayout->addWidget(_clear, 4, 1, 1, 1);

        line1 = new QFrame(KJSErrorDlgBase);
        line1->setObjectName(QString::fromUtf8("line1"));
        line1->setFrameShape(QFrame::HLine);
        line1->setFrameShadow(QFrame::Sunken);
        line1->setFrameShape(QFrame::HLine);

        gridLayout->addWidget(line1, 2, 0, 1, 3);

        spacerItem = new QSpacerItem(311, 21, QSizePolicy::Expanding, QSizePolicy::Minimum);

        gridLayout->addItem(spacerItem, 3, 0, 2, 1);

        _errorText = new QTextBrowser(KJSErrorDlgBase);
        _errorText->setObjectName(QString::fromUtf8("_errorText"));
        QSizePolicy sizePolicy(static_cast<QSizePolicy::Policy>(7), static_cast<QSizePolicy::Policy>(7));
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(1);
        sizePolicy.setHeightForWidth(_errorText->sizePolicy().hasHeightForWidth());
        _errorText->setSizePolicy(sizePolicy);
        _errorText->setAcceptRichText(false);

        gridLayout->addWidget(_errorText, 1, 0, 1, 3);

        _url = new QLabel(KJSErrorDlgBase);
        _url->setObjectName(QString::fromUtf8("_url"));

        gridLayout->addWidget(_url, 0, 0, 1, 3);

        QWidget::setTabOrder(_errorText, _close);
        QWidget::setTabOrder(_close, _clear);

        retranslateUi(KJSErrorDlgBase);

        QMetaObject::connectSlotsByName(KJSErrorDlgBase);
    } // setupUi

    void retranslateUi(QDialog *KJSErrorDlgBase)
    {
        KJSErrorDlgBase->setWindowTitle(tr2i18n("JavaScript Errors", 0));
#ifndef UI_QT_NO_WHATSTHIS
        KJSErrorDlgBase->setWhatsThis(tr2i18n("This dialog provides you with notification and details of scripting errors that occur on web pages.  In many cases it is due to an error in the web site as designed by its author.  In other cases it is the result of a programming error in Konqueror.  If you suspect the former, please contact the webmaster of the site in question.  Conversely if you suspect an error in Konqueror, please file a bug report at http://bugs.kde.org/.  A test case which illustrates the problem will be appreciated.", 0));
#endif // QT_NO_WHATSTHIS
        _close->setText(tr2i18n("&Close", 0));
        _clear->setText(tr2i18n("C&lear", 0));
    } // retranslateUi

};

namespace Ui {
    class KJSErrorDlgBase: public Ui_KJSErrorDlgBase {};
} // namespace Ui

QT_END_NAMESPACE

#endif // KJSERRORDLGBASE_H

