#include <kdialog.h>
#include <klocale.h>

/********************************************************************************
** Form generated from reading UI file 'knotifyconfigactionswidgetbase.ui'
**
** Created by: Qt User Interface Compiler version 4.8.7
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KNOTIFYCONFIGACTIONSWIDGETBASE_H
#define UI_KNOTIFYCONFIGACTIONSWIDGETBASE_H

#include <QtCore/QVariant>
#include <QtGui/QAction>
#include <QtGui/QApplication>
#include <QtGui/QButtonGroup>
#include <QtGui/QCheckBox>
#include <QtGui/QGridLayout>
#include <QtGui/QHeaderView>
#include <QtGui/QToolButton>
#include <QtGui/QWidget>
#include <klineedit.h>
#include "kcombobox.h"
#include "kurlrequester.h"

QT_BEGIN_NAMESPACE

class Ui_KNotifyConfigActionsWidgetBase
{
public:
    QGridLayout *gridLayout;
    QCheckBox *Sound_check;
    QToolButton *Sound_play;
    KUrlRequester *Sound_select;
    QCheckBox *Popup_check;
    QCheckBox *Logfile_check;
    KUrlRequester *Logfile_select;
    QCheckBox *Taskbar_check;
    QCheckBox *Execute_check;
    KUrlRequester *Execute_select;
    QCheckBox *KTTS_check;
    KComboBox *KTTS_combo;
    KLineEdit *KTTS_select;

    void setupUi(QWidget *KNotifyConfigActionsWidgetBase)
    {
        if (KNotifyConfigActionsWidgetBase->objectName().isEmpty())
            KNotifyConfigActionsWidgetBase->setObjectName(QString::fromUtf8("KNotifyConfigActionsWidgetBase"));
        KNotifyConfigActionsWidgetBase->resize(433, 190);
        gridLayout = new QGridLayout(KNotifyConfigActionsWidgetBase);
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        Sound_check = new QCheckBox(KNotifyConfigActionsWidgetBase);
        Sound_check->setObjectName(QString::fromUtf8("Sound_check"));

        gridLayout->addWidget(Sound_check, 0, 0, 1, 1);

        Sound_play = new QToolButton(KNotifyConfigActionsWidgetBase);
        Sound_play->setObjectName(QString::fromUtf8("Sound_play"));
        Sound_play->setEnabled(false);

        gridLayout->addWidget(Sound_play, 0, 1, 1, 1);

        Sound_select = new KUrlRequester(KNotifyConfigActionsWidgetBase);
        Sound_select->setObjectName(QString::fromUtf8("Sound_select"));
        Sound_select->setEnabled(false);
        QSizePolicy sizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(Sound_select->sizePolicy().hasHeightForWidth());
        Sound_select->setSizePolicy(sizePolicy);

        gridLayout->addWidget(Sound_select, 0, 2, 1, 2);

        Popup_check = new QCheckBox(KNotifyConfigActionsWidgetBase);
        Popup_check->setObjectName(QString::fromUtf8("Popup_check"));

        gridLayout->addWidget(Popup_check, 1, 0, 1, 4);

        Logfile_check = new QCheckBox(KNotifyConfigActionsWidgetBase);
        Logfile_check->setObjectName(QString::fromUtf8("Logfile_check"));

        gridLayout->addWidget(Logfile_check, 2, 0, 1, 1);

        Logfile_select = new KUrlRequester(KNotifyConfigActionsWidgetBase);
        Logfile_select->setObjectName(QString::fromUtf8("Logfile_select"));
        Logfile_select->setEnabled(false);
        sizePolicy.setHeightForWidth(Logfile_select->sizePolicy().hasHeightForWidth());
        Logfile_select->setSizePolicy(sizePolicy);

        gridLayout->addWidget(Logfile_select, 2, 1, 1, 3);

        Taskbar_check = new QCheckBox(KNotifyConfigActionsWidgetBase);
        Taskbar_check->setObjectName(QString::fromUtf8("Taskbar_check"));

        gridLayout->addWidget(Taskbar_check, 3, 0, 1, 4);

        Execute_check = new QCheckBox(KNotifyConfigActionsWidgetBase);
        Execute_check->setObjectName(QString::fromUtf8("Execute_check"));

        gridLayout->addWidget(Execute_check, 4, 0, 1, 1);

        Execute_select = new KUrlRequester(KNotifyConfigActionsWidgetBase);
        Execute_select->setObjectName(QString::fromUtf8("Execute_select"));
        Execute_select->setEnabled(false);
        sizePolicy.setHeightForWidth(Execute_select->sizePolicy().hasHeightForWidth());
        Execute_select->setSizePolicy(sizePolicy);

        gridLayout->addWidget(Execute_select, 4, 1, 1, 3);

        KTTS_check = new QCheckBox(KNotifyConfigActionsWidgetBase);
        KTTS_check->setObjectName(QString::fromUtf8("KTTS_check"));

        gridLayout->addWidget(KTTS_check, 5, 0, 1, 1);

        KTTS_combo = new KComboBox(KNotifyConfigActionsWidgetBase);
        KTTS_combo->setObjectName(QString::fromUtf8("KTTS_combo"));
        QSizePolicy sizePolicy1(QSizePolicy::Preferred, QSizePolicy::Preferred);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(KTTS_combo->sizePolicy().hasHeightForWidth());
        KTTS_combo->setSizePolicy(sizePolicy1);

        gridLayout->addWidget(KTTS_combo, 5, 1, 1, 2);

        KTTS_select = new KLineEdit(KNotifyConfigActionsWidgetBase);
        KTTS_select->setObjectName(QString::fromUtf8("KTTS_select"));

        gridLayout->addWidget(KTTS_select, 5, 3, 1, 1);


        retranslateUi(KNotifyConfigActionsWidgetBase);
        QObject::connect(Sound_check, SIGNAL(toggled(bool)), Sound_play, SLOT(setEnabled(bool)));
        QObject::connect(Sound_check, SIGNAL(toggled(bool)), Sound_select, SLOT(setEnabled(bool)));
        QObject::connect(Logfile_check, SIGNAL(toggled(bool)), Logfile_select, SLOT(setEnabled(bool)));
        QObject::connect(Execute_check, SIGNAL(toggled(bool)), Execute_select, SLOT(setEnabled(bool)));
        QObject::connect(KTTS_check, SIGNAL(toggled(bool)), KTTS_combo, SLOT(setEnabled(bool)));

        QMetaObject::connectSlotsByName(KNotifyConfigActionsWidgetBase);
    } // setupUi

    void retranslateUi(QWidget *KNotifyConfigActionsWidgetBase)
    {
        Sound_check->setText(tr2i18n("Play a &sound", 0));
        Sound_play->setText(QString());
#ifndef UI_QT_NO_TOOLTIP
        Sound_select->setToolTip(tr2i18n("Select the sound to play", 0));
#endif // QT_NO_TOOLTIP
        Popup_check->setText(tr2i18n("Show a message in a &popup", 0));
        Logfile_check->setText(tr2i18n("Log to a file", 0));
        Taskbar_check->setText(tr2i18n("Mark &taskbar entry", 0));
        Execute_check->setText(tr2i18n("Run &command", 0));
#ifndef UI_QT_NO_TOOLTIP
        Execute_select->setToolTip(tr2i18n("Select the command to run", 0));
#endif // QT_NO_TOOLTIP
        KTTS_check->setText(tr2i18n("Sp&eech", 0));
        KTTS_combo->clear();
        KTTS_combo->insertItems(0, QStringList()
         << tr2i18n("Speak Event Message", 0)
         << tr2i18n("Speak Event Name", 0)
         << tr2i18n("Speak Custom Text", 0)
        );
#ifndef UI_QT_NO_WHATSTHIS
        KTTS_combo->setWhatsThis(tr2i18n("<qt>Specifies how Jovie should speak the event when received.  If you select \"Speak custom text\", enter the text in the box.  You may use the following substitution strings in the text:<dl><dt>%e</dt><dd>Name of the event</dd><dt>%a</dt><dd>Application that sent the event</dd><dt>%m</dt><dd>The message sent by the application</dd></dl></qt>", 0));
#endif // QT_NO_WHATSTHIS
#ifndef UI_QT_NO_WHATSTHIS
        KTTS_select->setWhatsThis(tr2i18n("<qt>Specifies how Jovie should speak the event when received.  If you select \"Speak custom text\", enter the text in the box.  You may use the following substitution strings in the text:<dl><dt>%e</dt><dd>Name of the event</dd><dt>%a</dt><dd>Application that sent the event</dd><dt>%m</dt><dd>The message sent by the application</dd></dl></qt>", 0));
#endif // QT_NO_WHATSTHIS
        Q_UNUSED(KNotifyConfigActionsWidgetBase);
    } // retranslateUi

};

namespace Ui {
    class KNotifyConfigActionsWidgetBase: public Ui_KNotifyConfigActionsWidgetBase {};
} // namespace Ui

QT_END_NAMESPACE

#endif // KNOTIFYCONFIGACTIONSWIDGETBASE_H

