/*
 * This file was generated by dbusxml2cpp version 0.6
 * Command line was: dbusxml2cpp -m -p kdirnotify -- org.kde.KDirNotify.xml
 *
 * dbusxml2cpp is Copyright (C) 2006 Trolltech AS. All rights reserved.
 *
 * This is an auto-generated file.
 * This file may have been hand-edited. Look for HAND-EDIT comments
 * before re-generating it.
 */

#include "kdirnotify.h"
#include <kdbusconnectionpool.h> // HAND-EDIT

/*
 * Implementation of interface class OrgKdeKDirNotifyInterface
 */

OrgKdeKDirNotifyInterface::OrgKdeKDirNotifyInterface(const QString &service, const QString &path, const QDBusConnection &connection, QObject *parent)
    : QDBusAbstractInterface(service, path, staticInterfaceName(), connection, parent)
{
}

OrgKdeKDirNotifyInterface::~OrgKdeKDirNotifyInterface()
{
}

static void emitSignal(const QString &signalName, const QVariantList &args)
{
    QDBusMessage message =
        QDBusMessage::createSignal("/", QLatin1String(org::kde::KDirNotify::staticInterfaceName()), signalName);
    message.setArguments(args);
    // HAND-EDIT
    KDBusConnectionPool::threadConnection().send(message);
}

void OrgKdeKDirNotifyInterface::emitFileRenamed(const QString &src, const QString &dst)
{
    emitSignal(QLatin1String("FileRenamed"), QVariantList() << src << dst);
}

void OrgKdeKDirNotifyInterface::emitFileMoved(const QString &src, const QString &dst)
{
    emitSignal(QLatin1String("FileMoved"), QVariantList() << src << dst);
}

void OrgKdeKDirNotifyInterface::emitFilesAdded(const QString &directory)
{
    emitSignal(QLatin1String("FilesAdded"), QVariantList() << directory);
}

void OrgKdeKDirNotifyInterface::emitFilesChanged(const QStringList &fileList)
{
    emitSignal(QLatin1String("FilesChanged"), QVariantList() << QVariant(fileList));
}

void OrgKdeKDirNotifyInterface::emitFilesRemoved(const QStringList &fileList)
{
    emitSignal(QLatin1String("FilesRemoved"), QVariantList() << QVariant(fileList));
}

void OrgKdeKDirNotifyInterface::emitEnteredDirectory(const QString &url)
{
    emitSignal(QLatin1String("enteredDirectory"), QVariantList() << url);
}

void OrgKdeKDirNotifyInterface::emitLeftDirectory(const QString &url)
{
    emitSignal(QLatin1String("leftDirectory"), QVariantList() << url);
}

#include "kdirnotify.moc"
