/*
 * This file was generated by qdbusxml2cpp version 0.7
 * Command line was: qdbusxml2cpp -m -a authadaptor -i auth/backends/dbus/DBusHelperProxy.h -l KAuth::DBusHelperProxy /usr/src/ports/kdelibs/kdelibs-4.14.38-1.x86_64/src/kdelibs-4.14.38/kdecore/auth/backends/dbus/org.kde.auth.xml
 *
 * qdbusxml2cpp is Copyright (C) 2015 The Qt Company Ltd.
 *
 * This is an auto-generated file.
 * Do not edit! All changes made to it will be lost.
 */

#include "authadaptor.h"
#include <QtCore/QMetaObject>
#include <QtCore/QByteArray>
#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QVariant>

/*
 * Implementation of adaptor class AuthAdaptor
 */

AuthAdaptor::AuthAdaptor(KAuth::DBusHelperProxy *parent)
    : QDBusAbstractAdaptor(parent)
{
    // constructor
    setAutoRelaySignals(true);
}

AuthAdaptor::~AuthAdaptor()
{
    // destructor
}

uint AuthAdaptor::authorizeAction(const QString &action, const QByteArray &callerID)
{
    // handle method call org.kde.auth.authorizeAction
    return parent()->authorizeAction(action, callerID);
}

QByteArray AuthAdaptor::performAction(const QString &action, const QByteArray &callerID, const QByteArray &arguments)
{
    // handle method call org.kde.auth.performAction
    return parent()->performAction(action, callerID, arguments);
}

void AuthAdaptor::performActions(const QByteArray &blob, const QByteArray &callerID)
{
    // handle method call org.kde.auth.performActions
    parent()->performActions(blob, callerID);
}

void AuthAdaptor::stopAction(const QString &action)
{
    // handle method call org.kde.auth.stopAction
    parent()->stopAction(action);
}


#include "authadaptor.moc"
