#include <kdialog.h>
#include <klocale.h>

/********************************************************************************
** Form generated from reading UI file 'DownloadDialog.ui'
**
** Created by: Qt User Interface Compiler version 4.8.7
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_DOWNLOADDIALOG_H
#define UI_DOWNLOADDIALOG_H

#include <QtCore/QVariant>
#include <QtGui/QAction>
#include <QtGui/QApplication>
#include <QtGui/QButtonGroup>
#include <QtGui/QDialogButtonBox>
#include <QtGui/QHBoxLayout>
#include <QtGui/QHeaderView>
#include <QtGui/QLabel>
#include <QtGui/QListView>
#include <QtGui/QSpacerItem>
#include <QtGui/QToolButton>
#include <QtGui/QVBoxLayout>
#include <QtGui/QWidget>
#include "kcombobox.h"
#include "klineedit.h"
#include "ktitlewidget.h"
#include "ui/qprogressindicator.h"

QT_BEGIN_NAMESPACE

class Ui_DownloadDialog
{
public:
    QVBoxLayout *verticalLayout_2;
    KTitleWidget *m_titleWidget;
    QHBoxLayout *hboxLayout;
    QHBoxLayout *hboxLayout1;
    QLabel *label;
    QLabel *m_providerLinkLabel;
    KComboBox *m_sourceCombo;
    QHBoxLayout *hboxLayout2;
    QLabel *label_2;
    KComboBox *m_sortCombo;
    KLineEdit *m_searchEdit;
    QVBoxLayout *verticalLayout;
    QListView *m_listView;
    QHBoxLayout *hboxLayout3;
    QProgressIndicator *m_progress;
    QToolButton *m_collaborationButton;
    QSpacerItem *horizontalSpacer;
    QDialogButtonBox *m_buttonBox;

    void setupUi(QWidget *DownloadDialog)
    {
        if (DownloadDialog->objectName().isEmpty())
            DownloadDialog->setObjectName(QString::fromUtf8("DownloadDialog"));
        DownloadDialog->resize(725, 487);
        verticalLayout_2 = new QVBoxLayout(DownloadDialog);
        verticalLayout_2->setContentsMargins(0, 0, 0, 0);
        verticalLayout_2->setObjectName(QString::fromUtf8("verticalLayout_2"));
        m_titleWidget = new KTitleWidget(DownloadDialog);
        m_titleWidget->setObjectName(QString::fromUtf8("m_titleWidget"));

        verticalLayout_2->addWidget(m_titleWidget);

        hboxLayout = new QHBoxLayout();
        hboxLayout->setObjectName(QString::fromUtf8("hboxLayout"));
        hboxLayout1 = new QHBoxLayout();
        hboxLayout1->setObjectName(QString::fromUtf8("hboxLayout1"));
        label = new QLabel(DownloadDialog);
        label->setObjectName(QString::fromUtf8("label"));
        label->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        hboxLayout1->addWidget(label);

        m_providerLinkLabel = new QLabel(DownloadDialog);
        m_providerLinkLabel->setObjectName(QString::fromUtf8("m_providerLinkLabel"));
        m_providerLinkLabel->setOpenExternalLinks(true);

        hboxLayout1->addWidget(m_providerLinkLabel);

        m_sourceCombo = new KComboBox(DownloadDialog);
        m_sourceCombo->setObjectName(QString::fromUtf8("m_sourceCombo"));
        QSizePolicy sizePolicy(QSizePolicy::Preferred, QSizePolicy::Fixed);
        sizePolicy.setHorizontalStretch(1);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(m_sourceCombo->sizePolicy().hasHeightForWidth());
        m_sourceCombo->setSizePolicy(sizePolicy);

        hboxLayout1->addWidget(m_sourceCombo);


        hboxLayout->addLayout(hboxLayout1);

        hboxLayout2 = new QHBoxLayout();
        hboxLayout2->setObjectName(QString::fromUtf8("hboxLayout2"));
        label_2 = new QLabel(DownloadDialog);
        label_2->setObjectName(QString::fromUtf8("label_2"));
        label_2->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        hboxLayout2->addWidget(label_2);

        m_sortCombo = new KComboBox(DownloadDialog);
        m_sortCombo->setObjectName(QString::fromUtf8("m_sortCombo"));
        sizePolicy.setHeightForWidth(m_sortCombo->sizePolicy().hasHeightForWidth());
        m_sortCombo->setSizePolicy(sizePolicy);

        hboxLayout2->addWidget(m_sortCombo);

        m_searchEdit = new KLineEdit(DownloadDialog);
        m_searchEdit->setObjectName(QString::fromUtf8("m_searchEdit"));
        sizePolicy.setHeightForWidth(m_searchEdit->sizePolicy().hasHeightForWidth());
        m_searchEdit->setSizePolicy(sizePolicy);
        m_searchEdit->setProperty("showClearButton", QVariant(true));

        hboxLayout2->addWidget(m_searchEdit);


        hboxLayout->addLayout(hboxLayout2);


        verticalLayout_2->addLayout(hboxLayout);

        verticalLayout = new QVBoxLayout();
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        m_listView = new QListView(DownloadDialog);
        m_listView->setObjectName(QString::fromUtf8("m_listView"));
        m_listView->setMouseTracking(true);
        m_listView->setVerticalScrollMode(QAbstractItemView::ScrollPerPixel);

        verticalLayout->addWidget(m_listView);


        verticalLayout_2->addLayout(verticalLayout);

        hboxLayout3 = new QHBoxLayout();
        hboxLayout3->setObjectName(QString::fromUtf8("hboxLayout3"));
        m_progress = new QProgressIndicator(DownloadDialog);
        m_progress->setObjectName(QString::fromUtf8("m_progress"));
        QSizePolicy sizePolicy1(QSizePolicy::Preferred, QSizePolicy::Preferred);
        sizePolicy1.setHorizontalStretch(1);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(m_progress->sizePolicy().hasHeightForWidth());
        m_progress->setSizePolicy(sizePolicy1);
        m_progress->setFrameShape(QFrame::NoFrame);
        m_progress->setFrameShadow(QFrame::Plain);

        hboxLayout3->addWidget(m_progress);

        m_collaborationButton = new QToolButton(DownloadDialog);
        m_collaborationButton->setObjectName(QString::fromUtf8("m_collaborationButton"));
        m_collaborationButton->setEnabled(false);
        m_collaborationButton->setPopupMode(QToolButton::MenuButtonPopup);
        m_collaborationButton->setToolButtonStyle(Qt::ToolButtonTextBesideIcon);

        hboxLayout3->addWidget(m_collaborationButton);

        horizontalSpacer = new QSpacerItem(QSizePolicy::Expanding, QSizePolicy::Minimum);

        hboxLayout3->addItem(horizontalSpacer);

        m_buttonBox = new QDialogButtonBox(DownloadDialog);
        m_buttonBox->setObjectName(QString::fromUtf8("m_buttonBox"));
        QSizePolicy sizePolicy2(QSizePolicy::Preferred, QSizePolicy::Fixed);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(m_buttonBox->sizePolicy().hasHeightForWidth());
        m_buttonBox->setSizePolicy(sizePolicy2);
        m_buttonBox->setOrientation(Qt::Horizontal);
        m_buttonBox->setStandardButtons(QDialogButtonBox::Close);

        hboxLayout3->addWidget(m_buttonBox);


        verticalLayout_2->addLayout(hboxLayout3);

#ifndef UI_QT_NO_SHORTCUT
        label->setBuddy(m_sourceCombo);
        label_2->setBuddy(m_sortCombo);
#endif // QT_NO_SHORTCUT

        retranslateUi(DownloadDialog);

        QMetaObject::connectSlotsByName(DownloadDialog);
    } // setupUi

    void retranslateUi(QWidget *DownloadDialog)
    {
        label->setText(tr2i18n("&Source:", 0));
        m_providerLinkLabel->setText(tr2i18n("?", 0));
        label_2->setText(tr2i18n("&Order by:", 0));
        m_searchEdit->setClickMessage(tr2i18n("Enter search phrase here", 0));
        m_collaborationButton->setText(tr2i18n("Collaborate", 0));
        Q_UNUSED(DownloadDialog);
    } // retranslateUi

};

namespace Ui {
    class DownloadDialog: public Ui_DownloadDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // DOWNLOADDIALOG_H

