/*
 * This file was generated by qdbusxml2cpp version 0.7
 * Command line was: qdbusxml2cpp -m -a fileundomanager_adaptor -c KIOFileUndoManagerAdaptor -i fileundomanager_p.h -l KIO::FileUndoManagerPrivate /usr/src/ports/kdelibs/kdelibs-4.14.38-1.x86_64/src/kdelibs-4.14.38/kio/kio/org.kde.kio.FileUndoManager.xml
 *
 * qdbusxml2cpp is Copyright (C) 2015 The Qt Company Ltd.
 *
 * This is an auto-generated file.
 * Do not edit! All changes made to it will be lost.
 */

#include "fileundomanager_adaptor.h"
#include <QtCore/QMetaObject>
#include <QtCore/QByteArray>
#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QVariant>

/*
 * Implementation of adaptor class KIOFileUndoManagerAdaptor
 */

KIOFileUndoManagerAdaptor::KIOFileUndoManagerAdaptor(KIO::FileUndoManagerPrivate *parent)
    : QDBusAbstractAdaptor(parent)
{
    // constructor
    setAutoRelaySignals(true);
}

KIOFileUndoManagerAdaptor::~KIOFileUndoManagerAdaptor()
{
    // destructor
}

QByteArray KIOFileUndoManagerAdaptor::get()
{
    // handle method call org.kde.libkonq.FileUndoManager.get
    return parent()->get();
}


#include "fileundomanager_adaptor.moc"
