#include <kdialog.h>
#include <klocale.h>

/********************************************************************************
** Form generated from reading UI file 'knewpassworddialog.ui'
**
** Created by: Qt User Interface Compiler version 4.8.7
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KNEWPASSWORDDIALOG_H
#define UI_KNEWPASSWORDDIALOG_H

#include <QtCore/QVariant>
#include <QtGui/QAction>
#include <QtGui/QApplication>
#include <QtGui/QButtonGroup>
#include <QtGui/QFormLayout>
#include <QtGui/QHBoxLayout>
#include <QtGui/QHeaderView>
#include <QtGui/QLabel>
#include <QtGui/QProgressBar>
#include <QtGui/QSpacerItem>
#include <QtGui/QVBoxLayout>
#include <QtGui/QWidget>
#include "klineedit.h"
#include "ktitlewidget.h"

QT_BEGIN_NAMESPACE

class Ui_KNewPasswordDialog
{
public:
    QVBoxLayout *verticalLayout;
    QHBoxLayout *horizontalLayout_2;
    QLabel *labelIcon;
    QLabel *labelPrompt;
    QFormLayout *formLayout;
    QLabel *labelPassword;
    QLabel *label_2;
    KLineEdit *linePassword;
    KLineEdit *lineVerifyPassword;
    QHBoxLayout *horizontalLayout;
    QLabel *labelStrengthMeter;
    QProgressBar *strengthBar;
    KTitleWidget *labelMatch;
    QSpacerItem *verticalSpacer;

    void setupUi(QWidget *KNewPasswordDialog)
    {
        if (KNewPasswordDialog->objectName().isEmpty())
            KNewPasswordDialog->setObjectName(QString::fromUtf8("KNewPasswordDialog"));
        verticalLayout = new QVBoxLayout(KNewPasswordDialog);
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName(QString::fromUtf8("horizontalLayout_2"));
        labelIcon = new QLabel(KNewPasswordDialog);
        labelIcon->setObjectName(QString::fromUtf8("labelIcon"));
        QSizePolicy sizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(labelIcon->sizePolicy().hasHeightForWidth());
        labelIcon->setSizePolicy(sizePolicy);
        labelIcon->setMinimumSize(QSize(96, 96));
        labelIcon->setMaximumSize(QSize(96, 96));

        horizontalLayout_2->addWidget(labelIcon);

        labelPrompt = new QLabel(KNewPasswordDialog);
        labelPrompt->setObjectName(QString::fromUtf8("labelPrompt"));
        labelPrompt->setWordWrap(true);

        horizontalLayout_2->addWidget(labelPrompt);


        verticalLayout->addLayout(horizontalLayout_2);

        formLayout = new QFormLayout();
        formLayout->setObjectName(QString::fromUtf8("formLayout"));
        labelPassword = new QLabel(KNewPasswordDialog);
        labelPassword->setObjectName(QString::fromUtf8("labelPassword"));

        formLayout->setWidget(0, QFormLayout::LabelRole, labelPassword);

        label_2 = new QLabel(KNewPasswordDialog);
        label_2->setObjectName(QString::fromUtf8("label_2"));

        formLayout->setWidget(1, QFormLayout::LabelRole, label_2);

        linePassword = new KLineEdit(KNewPasswordDialog);
        linePassword->setObjectName(QString::fromUtf8("linePassword"));
        linePassword->setPasswordMode(true);

        formLayout->setWidget(0, QFormLayout::FieldRole, linePassword);

        lineVerifyPassword = new KLineEdit(KNewPasswordDialog);
        lineVerifyPassword->setObjectName(QString::fromUtf8("lineVerifyPassword"));
        lineVerifyPassword->setPasswordMode(true);

        formLayout->setWidget(1, QFormLayout::FieldRole, lineVerifyPassword);


        verticalLayout->addLayout(formLayout);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName(QString::fromUtf8("horizontalLayout"));
        labelStrengthMeter = new QLabel(KNewPasswordDialog);
        labelStrengthMeter->setObjectName(QString::fromUtf8("labelStrengthMeter"));

        horizontalLayout->addWidget(labelStrengthMeter);

        strengthBar = new QProgressBar(KNewPasswordDialog);
        strengthBar->setObjectName(QString::fromUtf8("strengthBar"));
        strengthBar->setValue(0);
        strengthBar->setTextVisible(false);

        horizontalLayout->addWidget(strengthBar);


        verticalLayout->addLayout(horizontalLayout);

        labelMatch = new KTitleWidget(KNewPasswordDialog);
        labelMatch->setObjectName(QString::fromUtf8("labelMatch"));

        verticalLayout->addWidget(labelMatch);

        verticalSpacer = new QSpacerItem(0, 0, QSizePolicy::Minimum, QSizePolicy::Expanding);

        verticalLayout->addItem(verticalSpacer);

#ifndef UI_QT_NO_SHORTCUT
        labelPassword->setBuddy(linePassword);
        label_2->setBuddy(lineVerifyPassword);
        labelStrengthMeter->setBuddy(strengthBar);
#endif // QT_NO_SHORTCUT

        retranslateUi(KNewPasswordDialog);

        QMetaObject::connectSlotsByName(KNewPasswordDialog);
    } // setupUi

    void retranslateUi(QWidget *KNewPasswordDialog)
    {
        labelPassword->setText(tr2i18n("Password:", 0));
        label_2->setText(tr2i18n("&Verify:", 0));
        labelStrengthMeter->setText(tr2i18n("Password strength meter:", 0));
        labelMatch->setText(tr2i18n("Passwords do not match", 0));
        Q_UNUSED(KNewPasswordDialog);
    } // retranslateUi

};

namespace Ui {
    class KNewPasswordDialog: public Ui_KNewPasswordDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // KNEWPASSWORDDIALOG_H

