/****************************************************************************
** Meta object code from reading C++ file 'internetgateway.h'
**
** Created by: The Qt Meta Object Compiler version 63 (Qt 4.8.7)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../src/kdelibs-4.14.38/solid/solid/internetgateway.h"
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'internetgateway.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 63
#error "This file was generated using the moc from 4.8.7. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
static const uint qt_meta_data_Solid__InternetGateway[] = {

 // content:
       6,       // revision
       0,       // classname
       0,    0, // classinfo
       4,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       4,       // signalCount

 // signals: signature, parameters, type, tag, flags
      92,   24,   23,   23, 0x05,
     196,  156,   23,   23, 0x05,
     255,  247,   23,   23, 0x05,
     299,  280,   23,   23, 0x05,

       0        // eod
};

static const char qt_meta_stringdata_Solid__InternetGateway[] = {
    "Solid::InternetGateway\0\0"
    "remoteHost,externalPort,mappingProtocol,internalPort,internalClient\0"
    "portMappingAdded(QString,qint16,NetworkProtocol,qint16,QString)\0"
    "remoteHost,externalPort,mappingProtocol\0"
    "portMappingDeleted(QString,qint16,NetworkProtocol)\0"
    "enabled\0enabledForInternet(bool)\0"
    "currentConnections\0"
    "currentConnectionsDataIsReady(QStringList)\0"
};

void Solid::InternetGateway::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        Q_ASSERT(staticMetaObject.cast(_o));
        InternetGateway *_t = static_cast<InternetGateway *>(_o);
        switch (_id) {
        case 0: _t->portMappingAdded((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< qint16(*)>(_a[2])),(*reinterpret_cast< const NetworkProtocol(*)>(_a[3])),(*reinterpret_cast< qint16(*)>(_a[4])),(*reinterpret_cast< const QString(*)>(_a[5]))); break;
        case 1: _t->portMappingDeleted((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< qint16(*)>(_a[2])),(*reinterpret_cast< const NetworkProtocol(*)>(_a[3]))); break;
        case 2: _t->enabledForInternet((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 3: _t->currentConnectionsDataIsReady((*reinterpret_cast< QStringList(*)>(_a[1]))); break;
        default: ;
        }
    }
}

const QMetaObjectExtraData Solid::InternetGateway::staticMetaObjectExtraData = {
    0,  qt_static_metacall 
};

const QMetaObject Solid::InternetGateway::staticMetaObject = {
    { &Solid::DeviceInterface::staticMetaObject, qt_meta_stringdata_Solid__InternetGateway,
      qt_meta_data_Solid__InternetGateway, &staticMetaObjectExtraData }
};

#ifdef Q_NO_DATA_RELOCATION
const QMetaObject &Solid::InternetGateway::getStaticMetaObject() { return staticMetaObject; }
#endif //Q_NO_DATA_RELOCATION

const QMetaObject *Solid::InternetGateway::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->metaObject : &staticMetaObject;
}

void *Solid::InternetGateway::qt_metacast(const char *_clname)
{
    if (!_clname) return 0;
    if (!strcmp(_clname, qt_meta_stringdata_Solid__InternetGateway))
        return static_cast<void*>(const_cast< InternetGateway*>(this));
    typedef Solid::DeviceInterface QMocSuperClass;
    return QMocSuperClass::qt_metacast(_clname);
}

int Solid::InternetGateway::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    typedef Solid::DeviceInterface QMocSuperClass;
    _id = QMocSuperClass::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    return _id;
}

// SIGNAL 0
void Solid::InternetGateway::portMappingAdded(const QString & _t1, qint16 _t2, const NetworkProtocol & _t3, qint16 _t4, const QString & _t5)
{
    void *_a[] = { 0, const_cast<void*>(reinterpret_cast<const void*>(&_t1)), const_cast<void*>(reinterpret_cast<const void*>(&_t2)), const_cast<void*>(reinterpret_cast<const void*>(&_t3)), const_cast<void*>(reinterpret_cast<const void*>(&_t4)), const_cast<void*>(reinterpret_cast<const void*>(&_t5)) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void Solid::InternetGateway::portMappingDeleted(const QString & _t1, qint16 _t2, const NetworkProtocol & _t3)
{
    void *_a[] = { 0, const_cast<void*>(reinterpret_cast<const void*>(&_t1)), const_cast<void*>(reinterpret_cast<const void*>(&_t2)), const_cast<void*>(reinterpret_cast<const void*>(&_t3)) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void Solid::InternetGateway::enabledForInternet(bool _t1)
{
    void *_a[] = { 0, const_cast<void*>(reinterpret_cast<const void*>(&_t1)) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void Solid::InternetGateway::currentConnectionsDataIsReady(QStringList _t1)
{
    void *_a[] = { 0, const_cast<void*>(reinterpret_cast<const void*>(&_t1)) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}
QT_END_MOC_NAMESPACE
