#include <kdialog.h>
#include <klocale.h>

/********************************************************************************
** Form generated from reading UI file 'kcupsoptionspageswidget.ui'
**
** Created by: Qt User Interface Compiler version 4.8.7
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KCUPSOPTIONSPAGESWIDGET_H
#define UI_KCUPSOPTIONSPAGESWIDGET_H

#include <QtCore/QVariant>
#include <QtGui/QAction>
#include <QtGui/QApplication>
#include <QtGui/QButtonGroup>
#include <QtGui/QCheckBox>
#include <QtGui/QGridLayout>
#include <QtGui/QGroupBox>
#include <QtGui/QHeaderView>
#include <QtGui/QLabel>
#include <QtGui/QRadioButton>
#include <QtGui/QSpacerItem>
#include <QtGui/QWidget>
#include "kbuttongroup.h"
#include "kcombobox.h"
#include "klineedit.h"

QT_BEGIN_NAMESPACE

class Ui_KCupsOptionsPagesWidget
{
public:
    QGridLayout *gridLayout_7;
    KButtonGroup *pagesPerSheetButtonGroup;
    QGridLayout *gridLayout;
    QRadioButton *oneUpRadioButton;
    QRadioButton *sixUpRadioButton;
    QRadioButton *twoUpRadioButton;
    QRadioButton *nineUpRadioButton;
    QRadioButton *fourUpRadioButton;
    QRadioButton *sixteenUpRadioButton;
    KComboBox *pagesPerSheetLayoutCombo;
    QSpacerItem *verticalSpacer_2;
    QGroupBox *bannerPagesGroupBox;
    QGridLayout *gridLayout_3;
    QLabel *startBannerLabel;
    KComboBox *startBannerPageCombo;
    QLabel *endBannerLabel;
    KComboBox *endBannerPageCombo;
    QSpacerItem *verticalSpacer;
    QGroupBox *pageLabelGroupBox;
    QGridLayout *gridLayout_6;
    KLineEdit *pageLabelLineEdit;
    QSpacerItem *verticalSpacer_5;
    QGroupBox *groupBox;
    QGridLayout *gridLayout_5;
    KComboBox *pageBorderCombo;
    QSpacerItem *verticalSpacer_4;
    QGroupBox *mirrorPagesGroupBox;
    QGridLayout *gridLayout_4;
    QCheckBox *mirrorPagesCheckBox;
    QSpacerItem *verticalSpacer_3;

    void setupUi(QWidget *KCupsOptionsPagesWidget)
    {
        if (KCupsOptionsPagesWidget->objectName().isEmpty())
            KCupsOptionsPagesWidget->setObjectName(QString::fromUtf8("KCupsOptionsPagesWidget"));
        KCupsOptionsPagesWidget->resize(548, 319);
        gridLayout_7 = new QGridLayout(KCupsOptionsPagesWidget);
        gridLayout_7->setObjectName(QString::fromUtf8("gridLayout_7"));
        pagesPerSheetButtonGroup = new KButtonGroup(KCupsOptionsPagesWidget);
        pagesPerSheetButtonGroup->setObjectName(QString::fromUtf8("pagesPerSheetButtonGroup"));
        gridLayout = new QGridLayout(pagesPerSheetButtonGroup);
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        oneUpRadioButton = new QRadioButton(pagesPerSheetButtonGroup);
        oneUpRadioButton->setObjectName(QString::fromUtf8("oneUpRadioButton"));
        oneUpRadioButton->setChecked(true);

        gridLayout->addWidget(oneUpRadioButton, 0, 0, 1, 1);

        sixUpRadioButton = new QRadioButton(pagesPerSheetButtonGroup);
        sixUpRadioButton->setObjectName(QString::fromUtf8("sixUpRadioButton"));

        gridLayout->addWidget(sixUpRadioButton, 0, 1, 1, 1);

        twoUpRadioButton = new QRadioButton(pagesPerSheetButtonGroup);
        twoUpRadioButton->setObjectName(QString::fromUtf8("twoUpRadioButton"));

        gridLayout->addWidget(twoUpRadioButton, 1, 0, 1, 1);

        nineUpRadioButton = new QRadioButton(pagesPerSheetButtonGroup);
        nineUpRadioButton->setObjectName(QString::fromUtf8("nineUpRadioButton"));

        gridLayout->addWidget(nineUpRadioButton, 1, 1, 1, 1);

        fourUpRadioButton = new QRadioButton(pagesPerSheetButtonGroup);
        fourUpRadioButton->setObjectName(QString::fromUtf8("fourUpRadioButton"));

        gridLayout->addWidget(fourUpRadioButton, 2, 0, 1, 1);

        sixteenUpRadioButton = new QRadioButton(pagesPerSheetButtonGroup);
        sixteenUpRadioButton->setObjectName(QString::fromUtf8("sixteenUpRadioButton"));

        gridLayout->addWidget(sixteenUpRadioButton, 2, 1, 1, 1);

        pagesPerSheetLayoutCombo = new KComboBox(pagesPerSheetButtonGroup);
        pagesPerSheetLayoutCombo->setObjectName(QString::fromUtf8("pagesPerSheetLayoutCombo"));

        gridLayout->addWidget(pagesPerSheetLayoutCombo, 3, 0, 1, 2);

        verticalSpacer_2 = new QSpacerItem(20, 25, QSizePolicy::Minimum, QSizePolicy::Expanding);

        gridLayout->addItem(verticalSpacer_2, 4, 0, 1, 2);


        gridLayout_7->addWidget(pagesPerSheetButtonGroup, 0, 0, 2, 1);

        bannerPagesGroupBox = new QGroupBox(KCupsOptionsPagesWidget);
        bannerPagesGroupBox->setObjectName(QString::fromUtf8("bannerPagesGroupBox"));
        gridLayout_3 = new QGridLayout(bannerPagesGroupBox);
        gridLayout_3->setObjectName(QString::fromUtf8("gridLayout_3"));
        startBannerLabel = new QLabel(bannerPagesGroupBox);
        startBannerLabel->setObjectName(QString::fromUtf8("startBannerLabel"));

        gridLayout_3->addWidget(startBannerLabel, 0, 0, 1, 1);

        startBannerPageCombo = new KComboBox(bannerPagesGroupBox);
        startBannerPageCombo->setObjectName(QString::fromUtf8("startBannerPageCombo"));

        gridLayout_3->addWidget(startBannerPageCombo, 0, 1, 1, 1);

        endBannerLabel = new QLabel(bannerPagesGroupBox);
        endBannerLabel->setObjectName(QString::fromUtf8("endBannerLabel"));

        gridLayout_3->addWidget(endBannerLabel, 1, 0, 1, 1);

        endBannerPageCombo = new KComboBox(bannerPagesGroupBox);
        endBannerPageCombo->setObjectName(QString::fromUtf8("endBannerPageCombo"));

        gridLayout_3->addWidget(endBannerPageCombo, 1, 1, 1, 1);

        verticalSpacer = new QSpacerItem(20, 33, QSizePolicy::Minimum, QSizePolicy::Expanding);

        gridLayout_3->addItem(verticalSpacer, 2, 0, 1, 2);


        gridLayout_7->addWidget(bannerPagesGroupBox, 0, 1, 1, 1);

        pageLabelGroupBox = new QGroupBox(KCupsOptionsPagesWidget);
        pageLabelGroupBox->setObjectName(QString::fromUtf8("pageLabelGroupBox"));
        gridLayout_6 = new QGridLayout(pageLabelGroupBox);
        gridLayout_6->setObjectName(QString::fromUtf8("gridLayout_6"));
        pageLabelLineEdit = new KLineEdit(pageLabelGroupBox);
        pageLabelLineEdit->setObjectName(QString::fromUtf8("pageLabelLineEdit"));

        gridLayout_6->addWidget(pageLabelLineEdit, 0, 0, 1, 1);

        verticalSpacer_5 = new QSpacerItem(20, 11, QSizePolicy::Minimum, QSizePolicy::Expanding);

        gridLayout_6->addItem(verticalSpacer_5, 1, 0, 1, 1);


        gridLayout_7->addWidget(pageLabelGroupBox, 1, 1, 1, 1);

        groupBox = new QGroupBox(KCupsOptionsPagesWidget);
        groupBox->setObjectName(QString::fromUtf8("groupBox"));
        gridLayout_5 = new QGridLayout(groupBox);
        gridLayout_5->setObjectName(QString::fromUtf8("gridLayout_5"));
        pageBorderCombo = new KComboBox(groupBox);
        pageBorderCombo->setObjectName(QString::fromUtf8("pageBorderCombo"));

        gridLayout_5->addWidget(pageBorderCombo, 0, 0, 1, 1);

        verticalSpacer_4 = new QSpacerItem(20, 2, QSizePolicy::Minimum, QSizePolicy::Expanding);

        gridLayout_5->addItem(verticalSpacer_4, 1, 0, 1, 1);


        gridLayout_7->addWidget(groupBox, 2, 0, 1, 1);

        mirrorPagesGroupBox = new QGroupBox(KCupsOptionsPagesWidget);
        mirrorPagesGroupBox->setObjectName(QString::fromUtf8("mirrorPagesGroupBox"));
        gridLayout_4 = new QGridLayout(mirrorPagesGroupBox);
        gridLayout_4->setObjectName(QString::fromUtf8("gridLayout_4"));
        mirrorPagesCheckBox = new QCheckBox(mirrorPagesGroupBox);
        mirrorPagesCheckBox->setObjectName(QString::fromUtf8("mirrorPagesCheckBox"));

        gridLayout_4->addWidget(mirrorPagesCheckBox, 0, 0, 1, 1);

        verticalSpacer_3 = new QSpacerItem(20, 14, QSizePolicy::Minimum, QSizePolicy::Expanding);

        gridLayout_4->addItem(verticalSpacer_3, 1, 0, 1, 1);


        gridLayout_7->addWidget(mirrorPagesGroupBox, 2, 1, 1, 1);


        retranslateUi(KCupsOptionsPagesWidget);

        QMetaObject::connectSlotsByName(KCupsOptionsPagesWidget);
    } // setupUi

    void retranslateUi(QWidget *KCupsOptionsPagesWidget)
    {
        KCupsOptionsPagesWidget->setWindowTitle(tr2i18n("Pages", 0));
        pagesPerSheetButtonGroup->setTitle(tr2i18n("Pages Per Sheet", 0));
        oneUpRadioButton->setText(tr2i18n("1", 0));
        sixUpRadioButton->setText(tr2i18n("6", 0));
        twoUpRadioButton->setText(tr2i18n("2", 0));
        nineUpRadioButton->setText(tr2i18n("9", 0));
        fourUpRadioButton->setText(tr2i18n("4", 0));
        sixteenUpRadioButton->setText(tr2i18n("16", 0));
        bannerPagesGroupBox->setTitle(tr2i18n("Banner Pages", 0));
        startBannerLabel->setText(tr2i18n("Start", "Banner page at start"));
        endBannerLabel->setText(tr2i18n("End", "Banner page at end"));
        pageLabelGroupBox->setTitle(tr2i18n("Page Label", 0));
        groupBox->setTitle(tr2i18n("Page Border", 0));
        mirrorPagesGroupBox->setTitle(tr2i18n("Mirror Pages", 0));
        mirrorPagesCheckBox->setText(tr2i18n("Mirror pages along vertical axis", 0));
    } // retranslateUi

};

namespace Ui {
    class KCupsOptionsPagesWidget: public Ui_KCupsOptionsPagesWidget {};
} // namespace Ui

QT_END_NAMESPACE

#endif // KCUPSOPTIONSPAGESWIDGET_H

