/****************************************************************************
** Meta object code from reading C++ file 'DBusHelperProxy.h'
**
** Created by: The Qt Meta Object Compiler version 63 (Qt 4.8.7)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../src/kdelibs-4.14.38/kdecore/auth/backends/dbus/DBusHelperProxy.h"
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'DBusHelperProxy.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 63
#error "This file was generated using the moc from 4.8.7. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
static const uint qt_meta_data_KAuth__DBusHelperProxy[] = {

 // content:
       6,       // revision
       0,       // classname
       0,    0, // classinfo
       6,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: signature, parameters, type, tag, flags
      41,   24,   23,   23, 0x05,

 // slots: signature, parameters, type, tag, flags
      85,   78,   23,   23, 0x0a,
     119,  105,   23,   23, 0x0a,
     194,  168,  157,   23, 0x0a,
     260,  244,  239,   23, 0x0a,
     296,   24,   23,   23, 0x08,

       0        // eod
};

static const char qt_meta_stringdata_KAuth__DBusHelperProxy[] = {
    "KAuth::DBusHelperProxy\0\0type,action,blob\0"
    "remoteSignal(int,QString,QByteArray)\0"
    "action\0stopAction(QString)\0blob,callerID\0"
    "performActions(QByteArray,QByteArray)\0"
    "QByteArray\0action,callerID,arguments\0"
    "performAction(QString,QByteArray,QByteArray)\0"
    "uint\0action,callerID\0"
    "authorizeAction(QString,QByteArray)\0"
    "remoteSignalReceived(int,QString,QByteArray)\0"
};

void KAuth::DBusHelperProxy::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        Q_ASSERT(staticMetaObject.cast(_o));
        DBusHelperProxy *_t = static_cast<DBusHelperProxy *>(_o);
        switch (_id) {
        case 0: _t->remoteSignal((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< const QByteArray(*)>(_a[3]))); break;
        case 1: _t->stopAction((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 2: _t->performActions((*reinterpret_cast< QByteArray(*)>(_a[1])),(*reinterpret_cast< const QByteArray(*)>(_a[2]))); break;
        case 3: { QByteArray _r = _t->performAction((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QByteArray(*)>(_a[2])),(*reinterpret_cast< QByteArray(*)>(_a[3])));
            if (_a[0]) *reinterpret_cast< QByteArray*>(_a[0]) = _r; }  break;
        case 4: { uint _r = _t->authorizeAction((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QByteArray(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< uint*>(_a[0]) = _r; }  break;
        case 5: _t->remoteSignalReceived((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< QByteArray(*)>(_a[3]))); break;
        default: ;
        }
    }
}

const QMetaObjectExtraData KAuth::DBusHelperProxy::staticMetaObjectExtraData = {
    0,  qt_static_metacall 
};

const QMetaObject KAuth::DBusHelperProxy::staticMetaObject = {
    { &HelperProxy::staticMetaObject, qt_meta_stringdata_KAuth__DBusHelperProxy,
      qt_meta_data_KAuth__DBusHelperProxy, &staticMetaObjectExtraData }
};

#ifdef Q_NO_DATA_RELOCATION
const QMetaObject &KAuth::DBusHelperProxy::getStaticMetaObject() { return staticMetaObject; }
#endif //Q_NO_DATA_RELOCATION

const QMetaObject *KAuth::DBusHelperProxy::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->metaObject : &staticMetaObject;
}

void *KAuth::DBusHelperProxy::qt_metacast(const char *_clname)
{
    if (!_clname) return 0;
    if (!strcmp(_clname, qt_meta_stringdata_KAuth__DBusHelperProxy))
        return static_cast<void*>(const_cast< DBusHelperProxy*>(this));
    if (!strcmp(_clname, "QDBusContext"))
        return static_cast< QDBusContext*>(const_cast< DBusHelperProxy*>(this));
    if (!strcmp(_clname, "org.kde.auth.HelperProxy/0.1"))
        return static_cast< KAuth::HelperProxy*>(const_cast< DBusHelperProxy*>(this));
    return HelperProxy::qt_metacast(_clname);
}

int KAuth::DBusHelperProxy::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = HelperProxy::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    return _id;
}

// SIGNAL 0
void KAuth::DBusHelperProxy::remoteSignal(int _t1, const QString & _t2, const QByteArray & _t3)
{
    void *_a[] = { 0, const_cast<void*>(reinterpret_cast<const void*>(&_t1)), const_cast<void*>(reinterpret_cast<const void*>(&_t2)), const_cast<void*>(reinterpret_cast<const void*>(&_t3)) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}
QT_END_MOC_NAMESPACE
