/**
* This file was autogenerated by makekdewidgets. Any changes will be lost!
* The generated code in this file is licensed under the same license that the
* input file.
*/
#include <QIcon>
#include <QtDesigner/QDesignerContainerExtension>
#include <QDesignerCustomWidgetInterface>
#include <qplugin.h>
#include <qdebug.h>

#include <kcomponentdata.h>
#include <classpreviews.h>
#include <ktimezone.h>
#include <keditlistbox.h>
#include <karrowbutton.h>


class KEditListBoxPlugin : public QObject, public QDesignerCustomWidgetInterface
{
	Q_OBJECT
	Q_INTERFACES(QDesignerCustomWidgetInterface)
public:
	KEditListBoxPlugin(QObject *parent = 0) :
		QObject(parent), mInitialized(false) {}
	virtual ~KEditListBoxPlugin() {}
	
	bool isContainer() const { return false; }
	bool isInitialized() const { return mInitialized; }
	QIcon icon() const { return QIcon(QLatin1String(":/pics/keditlistbox.png")); }
	QString codeTemplate() const { return QLatin1String("");}
	QString domXml() const { return QDesignerCustomWidgetInterface::domXml(); }
	QString group() const { return QLatin1String("Views (KDE)"); }
	QString includeFile() const { return QLatin1String("keditlistbox.h"); }
	QString name() const { return QLatin1String("KEditListBox"); }
	QString toolTip() const { return QLatin1String("Fullfeatured edit box with buttons (KDE)"); }
	QString whatsThis() const { return QLatin1String("KEditListBox Widget"); }

	QWidget* createWidget( QWidget* parent ) 
	{
		return new KEditListBox( parent );
	}
	void initialize(QDesignerFormEditorInterface *core) 
	{
		Q_UNUSED(core);
		if (mInitialized) return;
		mInitialized=true;
	}

private:
	bool mInitialized;
};


class KArrowButtonPlugin : public QObject, public QDesignerCustomWidgetInterface
{
	Q_OBJECT
	Q_INTERFACES(QDesignerCustomWidgetInterface)
public:
	KArrowButtonPlugin(QObject *parent = 0) :
		QObject(parent), mInitialized(false) {}
	virtual ~KArrowButtonPlugin() {}
	
	bool isContainer() const { return false; }
	bool isInitialized() const { return mInitialized; }
	QIcon icon() const { return QIcon(QLatin1String(":/pics/karrowbutton.png")); }
	QString codeTemplate() const { return QLatin1String("");}
	QString domXml() const { return QDesignerCustomWidgetInterface::domXml(); }
	QString group() const { return QLatin1String("Buttons (KDE)"); }
	QString includeFile() const { return QLatin1String("karrowbutton.h"); }
	QString name() const { return QLatin1String("KArrowButton"); }
	QString toolTip() const { return QLatin1String("Draws a button which shows an arrow pointing into a certain direction."); }
	QString whatsThis() const { return QLatin1String("KArrowButton Widget"); }

	QWidget* createWidget( QWidget* parent ) 
	{
		return new KArrowButton(parent,Qt::UpArrow);
	}
	void initialize(QDesignerFormEditorInterface *core) 
	{
		Q_UNUSED(core);
		if (mInitialized) return;
		mInitialized=true;
	}

private:
	bool mInitialized;
};


class KDE3SupportWidgets : public QObject, public QDesignerCustomWidgetCollectionInterface
{
	Q_OBJECT
	Q_INTERFACES(QDesignerCustomWidgetCollectionInterface)
public:
	KDE3SupportWidgets(QObject *parent = 0);
	virtual ~KDE3SupportWidgets() {}
	QList<QDesignerCustomWidgetInterface*> customWidgets() const { return m_plugins; } 
	
private:
	QList<QDesignerCustomWidgetInterface*> m_plugins;
};

Q_EXPORT_PLUGIN2(KDE3SupportWidgets, KDE3SupportWidgets)

KDE3SupportWidgets::KDE3SupportWidgets(QObject *parent)
	: QObject(parent){
	(void) new KComponentData("makekdewidgets");
		m_plugins.append( new KEditListBoxPlugin(this) );
		m_plugins.append( new KArrowButtonPlugin(this) );

}

#include </usr/src/ports/kdelibs/kdelibs-4.14.38-1.x86_64/build/kdewidgets/kdedeprecated.moc>

