#include <kdialog.h>
#include <klocale.h>

/********************************************************************************
** Form generated from reading UI file 'kpassworddialog.ui'
**
** Created by: Qt User Interface Compiler version 4.8.7
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KPASSWORDDIALOG_H
#define UI_KPASSWORDDIALOG_H

#include <QtCore/QVariant>
#include <QtGui/QAction>
#include <QtGui/QApplication>
#include <QtGui/QButtonGroup>
#include <QtGui/QCheckBox>
#include <QtGui/QFormLayout>
#include <QtGui/QHBoxLayout>
#include <QtGui/QHeaderView>
#include <QtGui/QLabel>
#include <QtGui/QLineEdit>
#include <QtGui/QRadioButton>
#include <QtGui/QSpacerItem>
#include <QtGui/QVBoxLayout>
#include <QtGui/QWidget>
#include "klineedit.h"
#include "ktitlewidget.h"

QT_BEGIN_NAMESPACE

class Ui_KPasswordDialog
{
public:
    QVBoxLayout *vboxLayout;
    QHBoxLayout *hboxLayout;
    QLabel *prompt;
    QSpacerItem *spacerItem;
    KTitleWidget *errorMessage;
    QRadioButton *anonymousRadioButton;
    QRadioButton *usePasswordButton;
    QWidget *credentialsGroup;
    QFormLayout *formLayout;
    QLabel *userNameLabel;
    QLineEdit *userEdit;
    QLabel *domainLabel;
    KLineEdit *domainEdit;
    QLabel *passwordLabel;
    KLineEdit *passEdit;
    QCheckBox *keepCheckBox;

    void setupUi(QWidget *KPasswordDialog)
    {
        if (KPasswordDialog->objectName().isEmpty())
            KPasswordDialog->setObjectName(QString::fromUtf8("KPasswordDialog"));
        KPasswordDialog->resize(400, 227);
        vboxLayout = new QVBoxLayout(KPasswordDialog);
        vboxLayout->setContentsMargins(0, 0, 0, 0);
        vboxLayout->setObjectName(QString::fromUtf8("vboxLayout"));
        hboxLayout = new QHBoxLayout();
        hboxLayout->setObjectName(QString::fromUtf8("hboxLayout"));
        prompt = new QLabel(KPasswordDialog);
        prompt->setObjectName(QString::fromUtf8("prompt"));
        QSizePolicy sizePolicy(QSizePolicy::Preferred, QSizePolicy::Preferred);
        sizePolicy.setHorizontalStretch(10);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(prompt->sizePolicy().hasHeightForWidth());
        prompt->setSizePolicy(sizePolicy);
        prompt->setTextFormat(Qt::RichText);
        prompt->setAlignment(Qt::AlignBottom|Qt::AlignLeading|Qt::AlignLeft);
        prompt->setWordWrap(true);

        hboxLayout->addWidget(prompt);


        vboxLayout->addLayout(hboxLayout);

        spacerItem = new QSpacerItem(400, 16, QSizePolicy::Minimum, QSizePolicy::Fixed);

        vboxLayout->addItem(spacerItem);

        errorMessage = new KTitleWidget(KPasswordDialog);
        errorMessage->setObjectName(QString::fromUtf8("errorMessage"));

        vboxLayout->addWidget(errorMessage);

        anonymousRadioButton = new QRadioButton(KPasswordDialog);
        anonymousRadioButton->setObjectName(QString::fromUtf8("anonymousRadioButton"));

        vboxLayout->addWidget(anonymousRadioButton);

        usePasswordButton = new QRadioButton(KPasswordDialog);
        usePasswordButton->setObjectName(QString::fromUtf8("usePasswordButton"));
        usePasswordButton->setChecked(true);

        vboxLayout->addWidget(usePasswordButton);

        credentialsGroup = new QWidget(KPasswordDialog);
        credentialsGroup->setObjectName(QString::fromUtf8("credentialsGroup"));
        formLayout = new QFormLayout(credentialsGroup);
        formLayout->setContentsMargins(1, 1, 1, 1);
        formLayout->setObjectName(QString::fromUtf8("formLayout"));
        userNameLabel = new QLabel(credentialsGroup);
        userNameLabel->setObjectName(QString::fromUtf8("userNameLabel"));

        formLayout->setWidget(1, QFormLayout::LabelRole, userNameLabel);

        userEdit = new QLineEdit(credentialsGroup);
        userEdit->setObjectName(QString::fromUtf8("userEdit"));

        formLayout->setWidget(1, QFormLayout::FieldRole, userEdit);

        domainLabel = new QLabel(credentialsGroup);
        domainLabel->setObjectName(QString::fromUtf8("domainLabel"));

        formLayout->setWidget(3, QFormLayout::LabelRole, domainLabel);

        domainEdit = new KLineEdit(credentialsGroup);
        domainEdit->setObjectName(QString::fromUtf8("domainEdit"));

        formLayout->setWidget(3, QFormLayout::FieldRole, domainEdit);

        passwordLabel = new QLabel(credentialsGroup);
        passwordLabel->setObjectName(QString::fromUtf8("passwordLabel"));

        formLayout->setWidget(4, QFormLayout::LabelRole, passwordLabel);

        passEdit = new KLineEdit(credentialsGroup);
        passEdit->setObjectName(QString::fromUtf8("passEdit"));
        passEdit->setPasswordMode(true);

        formLayout->setWidget(4, QFormLayout::FieldRole, passEdit);

        keepCheckBox = new QCheckBox(credentialsGroup);
        keepCheckBox->setObjectName(QString::fromUtf8("keepCheckBox"));

        formLayout->setWidget(5, QFormLayout::FieldRole, keepCheckBox);


        vboxLayout->addWidget(credentialsGroup);

        QWidget::setTabOrder(anonymousRadioButton, usePasswordButton);
        QWidget::setTabOrder(usePasswordButton, userEdit);
        QWidget::setTabOrder(userEdit, domainEdit);
        QWidget::setTabOrder(domainEdit, passEdit);
        QWidget::setTabOrder(passEdit, keepCheckBox);

        retranslateUi(KPasswordDialog);
        QObject::connect(usePasswordButton, SIGNAL(toggled(bool)), credentialsGroup, SLOT(setEnabled(bool)));
        QObject::connect(usePasswordButton, SIGNAL(clicked()), credentialsGroup, SLOT(setFocus()));

        QMetaObject::connectSlotsByName(KPasswordDialog);
    } // setupUi

    void retranslateUi(QWidget *KPasswordDialog)
    {
        prompt->setText(tr2i18n("Supply a username and password below.", 0));
        anonymousRadioButton->setText(tr2i18n("No password, use anonymous (or guest) login", 0));
        usePasswordButton->setText(tr2i18n("Use this password:", 0));
        userNameLabel->setText(tr2i18n("Username:", 0));
        domainLabel->setText(tr2i18n("Domain:", 0));
        passwordLabel->setText(tr2i18n("Password:", 0));
        keepCheckBox->setText(tr2i18n("Remember password", 0));
        Q_UNUSED(KPasswordDialog);
    } // retranslateUi

};

namespace Ui {
    class KPasswordDialog: public Ui_KPasswordDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // KPASSWORDDIALOG_H

