#include <kdialog.h>
#include <klocale.h>

/********************************************************************************
** Form generated from reading UI file 'kpropertiesdesktopadvbase.ui'
**
** Created by: Qt User Interface Compiler version 4.8.7
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KPROPERTIESDESKTOPADVBASE_H
#define UI_KPROPERTIESDESKTOPADVBASE_H

#include <QtCore/QVariant>
#include <QtGui/QAction>
#include <QtGui/QApplication>
#include <QtGui/QButtonGroup>
#include <QtGui/QCheckBox>
#include <QtGui/QGridLayout>
#include <QtGui/QGroupBox>
#include <QtGui/QHeaderView>
#include <QtGui/QLabel>
#include <QtGui/QSpacerItem>
#include <QtGui/QVBoxLayout>
#include <QtGui/QWidget>
#include "kcombobox.h"
#include "klineedit.h"

QT_BEGIN_NAMESPACE

class Ui_KPropertiesDesktopAdvBase
{
public:
    QVBoxLayout *vboxLayout;
    QGroupBox *buttonGroup2;
    QGridLayout *gridLayout;
    QSpacerItem *spacerItem;
    QCheckBox *terminalCheck;
    QLabel *terminalEditLabel;
    QCheckBox *terminalCloseCheck;
    KLineEdit *terminalEdit;
    QGroupBox *buttonGroup2_2;
    QGridLayout *gridLayout1;
    QCheckBox *suidCheck;
    QSpacerItem *spacerItem1;
    QLabel *suidEditLabel;
    KLineEdit *suidEdit;
    QGroupBox *buttonGroup4;
    QGridLayout *gridLayout2;
    QCheckBox *startupInfoCheck;
    QCheckBox *systrayCheck;
    QLabel *textLabel12;
    KComboBox *dbusCombo;
    QSpacerItem *spacerItem2;

    void setupUi(QWidget *KPropertiesDesktopAdvBase)
    {
        if (KPropertiesDesktopAdvBase->objectName().isEmpty())
            KPropertiesDesktopAdvBase->setObjectName(QString::fromUtf8("KPropertiesDesktopAdvBase"));
        vboxLayout = new QVBoxLayout(KPropertiesDesktopAdvBase);
        vboxLayout->setContentsMargins(0, 0, 0, 0);
        vboxLayout->setObjectName(QString::fromUtf8("vboxLayout"));
        buttonGroup2 = new QGroupBox(KPropertiesDesktopAdvBase);
        buttonGroup2->setObjectName(QString::fromUtf8("buttonGroup2"));
        gridLayout = new QGridLayout(buttonGroup2);
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        spacerItem = new QSpacerItem(20, 20, QSizePolicy::Minimum, QSizePolicy::Minimum);

        gridLayout->addItem(spacerItem, 1, 0, 2, 1);

        terminalCheck = new QCheckBox(buttonGroup2);
        terminalCheck->setObjectName(QString::fromUtf8("terminalCheck"));

        gridLayout->addWidget(terminalCheck, 0, 0, 1, 3);

        terminalEditLabel = new QLabel(buttonGroup2);
        terminalEditLabel->setObjectName(QString::fromUtf8("terminalEditLabel"));

        gridLayout->addWidget(terminalEditLabel, 2, 1, 1, 1);

        terminalCloseCheck = new QCheckBox(buttonGroup2);
        terminalCloseCheck->setObjectName(QString::fromUtf8("terminalCloseCheck"));

        gridLayout->addWidget(terminalCloseCheck, 1, 1, 1, 2);

        terminalEdit = new KLineEdit(buttonGroup2);
        terminalEdit->setObjectName(QString::fromUtf8("terminalEdit"));

        gridLayout->addWidget(terminalEdit, 2, 2, 1, 1);


        vboxLayout->addWidget(buttonGroup2);

        buttonGroup2_2 = new QGroupBox(KPropertiesDesktopAdvBase);
        buttonGroup2_2->setObjectName(QString::fromUtf8("buttonGroup2_2"));
        gridLayout1 = new QGridLayout(buttonGroup2_2);
        gridLayout1->setObjectName(QString::fromUtf8("gridLayout1"));
        suidCheck = new QCheckBox(buttonGroup2_2);
        suidCheck->setObjectName(QString::fromUtf8("suidCheck"));

        gridLayout1->addWidget(suidCheck, 0, 0, 1, 3);

        spacerItem1 = new QSpacerItem(20, 20, QSizePolicy::Minimum, QSizePolicy::Minimum);

        gridLayout1->addItem(spacerItem1, 1, 0, 1, 1);

        suidEditLabel = new QLabel(buttonGroup2_2);
        suidEditLabel->setObjectName(QString::fromUtf8("suidEditLabel"));

        gridLayout1->addWidget(suidEditLabel, 1, 1, 1, 1);

        suidEdit = new KLineEdit(buttonGroup2_2);
        suidEdit->setObjectName(QString::fromUtf8("suidEdit"));

        gridLayout1->addWidget(suidEdit, 1, 2, 1, 1);


        vboxLayout->addWidget(buttonGroup2_2);

        buttonGroup4 = new QGroupBox(KPropertiesDesktopAdvBase);
        buttonGroup4->setObjectName(QString::fromUtf8("buttonGroup4"));
        gridLayout2 = new QGridLayout(buttonGroup4);
        gridLayout2->setObjectName(QString::fromUtf8("gridLayout2"));
        startupInfoCheck = new QCheckBox(buttonGroup4);
        startupInfoCheck->setObjectName(QString::fromUtf8("startupInfoCheck"));

        gridLayout2->addWidget(startupInfoCheck, 0, 0, 1, 3);

        systrayCheck = new QCheckBox(buttonGroup4);
        systrayCheck->setObjectName(QString::fromUtf8("systrayCheck"));

        gridLayout2->addWidget(systrayCheck, 1, 0, 1, 3);

        textLabel12 = new QLabel(buttonGroup4);
        textLabel12->setObjectName(QString::fromUtf8("textLabel12"));

        gridLayout2->addWidget(textLabel12, 2, 0, 1, 1);

        dbusCombo = new KComboBox(buttonGroup4);
        dbusCombo->setObjectName(QString::fromUtf8("dbusCombo"));

        gridLayout2->addWidget(dbusCombo, 2, 1, 1, 1);

        spacerItem2 = new QSpacerItem(50, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        gridLayout2->addItem(spacerItem2, 2, 2, 1, 1);


        vboxLayout->addWidget(buttonGroup4);

#ifndef UI_QT_NO_SHORTCUT
        terminalEditLabel->setBuddy(terminalEdit);
        suidEditLabel->setBuddy(suidEdit);
        textLabel12->setBuddy(dbusCombo);
#endif // QT_NO_SHORTCUT

        retranslateUi(KPropertiesDesktopAdvBase);
        QObject::connect(terminalCheck, SIGNAL(toggled(bool)), terminalCloseCheck, SLOT(setEnabled(bool)));
        QObject::connect(terminalCheck, SIGNAL(toggled(bool)), terminalEdit, SLOT(setEnabled(bool)));
        QObject::connect(terminalCheck, SIGNAL(toggled(bool)), terminalEditLabel, SLOT(setEnabled(bool)));
        QObject::connect(suidCheck, SIGNAL(toggled(bool)), suidEdit, SLOT(setEnabled(bool)));
        QObject::connect(suidCheck, SIGNAL(toggled(bool)), suidEditLabel, SLOT(setEnabled(bool)));

        QMetaObject::connectSlotsByName(KPropertiesDesktopAdvBase);
    } // setupUi

    void retranslateUi(QWidget *KPropertiesDesktopAdvBase)
    {
        buttonGroup2->setTitle(tr2i18n("Terminal", "@title:group Title of a group that lets the user choose options about the terminal when launching a program"));
#ifndef UI_QT_NO_WHATSTHIS
        terminalCheck->setWhatsThis(tr2i18n("Check this option if the application you want to run is a text mode application or if you want the information that is provided by the terminal emulator window.", 0));
#endif // QT_NO_WHATSTHIS
        terminalCheck->setText(tr2i18n("&Run in terminal", 0));
        terminalEditLabel->setText(tr2i18n("&Terminal options:", 0));
#ifndef UI_QT_NO_WHATSTHIS
        terminalCloseCheck->setWhatsThis(tr2i18n("Check this option if the text mode application offers relevant information on exit. Keeping the terminal emulator open allows you to retrieve this information.", 0));
#endif // QT_NO_WHATSTHIS
        terminalCloseCheck->setText(tr2i18n("Do not &close when command exits", 0));
        buttonGroup2_2->setTitle(tr2i18n("User", "@title:group Title of a group that lets the user choose which user to use when launching a program"));
#ifndef UI_QT_NO_WHATSTHIS
        suidCheck->setWhatsThis(tr2i18n("Check this option if you want to run this application with a different user id. Every process has a different user id associated with it. This id code determines file access and other permissions. The password of the user is required to use this option.", 0));
#endif // QT_NO_WHATSTHIS
        suidCheck->setText(tr2i18n("Ru&n as a different user", 0));
#ifndef UI_QT_NO_WHATSTHIS
        suidEditLabel->setWhatsThis(tr2i18n("Enter the user name you want to run the application as.", 0));
#endif // QT_NO_WHATSTHIS
        suidEditLabel->setText(tr2i18n("&Username:", 0));
#ifndef UI_QT_NO_WHATSTHIS
        suidEdit->setWhatsThis(tr2i18n("Enter the user name you want to run the application as here.", 0));
#endif // QT_NO_WHATSTHIS
        buttonGroup4->setTitle(tr2i18n("Startup", "@title:group Title of a group that lets the user choose options regargin program startup"));
#ifndef UI_QT_NO_WHATSTHIS
        startupInfoCheck->setWhatsThis(tr2i18n("Check this option if you want to make clear that your application has started. This visual feedback may appear as a busy cursor or in the taskbar.", 0));
#endif // QT_NO_WHATSTHIS
        startupInfoCheck->setText(tr2i18n("Enable &launch feedback", 0));
#ifndef UI_QT_NO_WHATSTHIS
        systrayCheck->setWhatsThis(tr2i18n("Check this option if you want to have a system tray handle for your application.", 0));
#endif // QT_NO_WHATSTHIS
        systrayCheck->setText(tr2i18n("&Place in system tray", 0));
        textLabel12->setText(tr2i18n("&D-Bus registration:", 0));
        dbusCombo->clear();
        dbusCombo->insertItems(0, QStringList()
         << tr2i18n("None", 0)
         << tr2i18n("Multiple Instances", 0)
         << tr2i18n("Single Instance", 0)
         << tr2i18n("Run Until Finished", 0)
        );
        Q_UNUSED(KPropertiesDesktopAdvBase);
    } // retranslateUi

};

namespace Ui {
    class KPropertiesDesktopAdvBase: public Ui_KPropertiesDesktopAdvBase {};
} // namespace Ui

QT_END_NAMESPACE

#endif // KPROPERTIESDESKTOPADVBASE_H

