#include <kdialog.h>
#include <klocale.h>

/********************************************************************************
** Form generated from reading UI file 'kpropertiesdesktopbase.ui'
**
** Created by: Qt User Interface Compiler version 4.8.7
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KPROPERTIESDESKTOPBASE_H
#define UI_KPROPERTIESDESKTOPBASE_H

#include <QtCore/QVariant>
#include <QtGui/QAction>
#include <QtGui/QApplication>
#include <QtGui/QButtonGroup>
#include <QtGui/QFormLayout>
#include <QtGui/QHBoxLayout>
#include <QtGui/QHeaderView>
#include <QtGui/QLabel>
#include <QtGui/QPushButton>
#include <QtGui/QSpacerItem>
#include <QtGui/QTreeWidget>
#include <QtGui/QVBoxLayout>
#include <QtGui/QWidget>
#include "klineedit.h"
#include "kseparator.h"
#include "kurlrequester.h"

QT_BEGIN_NAMESPACE

class Ui_KPropertiesDesktopBase
{
public:
    QVBoxLayout *verticalLayout;
    QFormLayout *formLayout;
    QLabel *nameLabel;
    KLineEdit *nameEdit;
    QLabel *textLabel2;
    KLineEdit *genNameEdit;
    QLabel *textLabel3;
    KLineEdit *commentEdit;
    QLabel *textLabel4;
    QHBoxLayout *horizontalLayout;
    KLineEdit *commandEdit;
    QPushButton *browseButton;
    QLabel *textLabel5;
    KUrlRequester *pathEdit;
    KSeparator *kseparator;
    QLabel *textLabel7;
    QTreeWidget *filetypeList;
    QHBoxLayout *horizontalLayout_2;
    QPushButton *addFiletypeButton;
    QPushButton *delFiletypeButton;
    QSpacerItem *spacerItem;
    QPushButton *advancedButton;

    void setupUi(QWidget *KPropertiesDesktopBase)
    {
        if (KPropertiesDesktopBase->objectName().isEmpty())
            KPropertiesDesktopBase->setObjectName(QString::fromUtf8("KPropertiesDesktopBase"));
        verticalLayout = new QVBoxLayout(KPropertiesDesktopBase);
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        formLayout = new QFormLayout();
        formLayout->setObjectName(QString::fromUtf8("formLayout"));
        nameLabel = new QLabel(KPropertiesDesktopBase);
        nameLabel->setObjectName(QString::fromUtf8("nameLabel"));

        formLayout->setWidget(0, QFormLayout::LabelRole, nameLabel);

        nameEdit = new KLineEdit(KPropertiesDesktopBase);
        nameEdit->setObjectName(QString::fromUtf8("nameEdit"));

        formLayout->setWidget(0, QFormLayout::FieldRole, nameEdit);

        textLabel2 = new QLabel(KPropertiesDesktopBase);
        textLabel2->setObjectName(QString::fromUtf8("textLabel2"));

        formLayout->setWidget(1, QFormLayout::LabelRole, textLabel2);

        genNameEdit = new KLineEdit(KPropertiesDesktopBase);
        genNameEdit->setObjectName(QString::fromUtf8("genNameEdit"));

        formLayout->setWidget(1, QFormLayout::FieldRole, genNameEdit);

        textLabel3 = new QLabel(KPropertiesDesktopBase);
        textLabel3->setObjectName(QString::fromUtf8("textLabel3"));

        formLayout->setWidget(2, QFormLayout::LabelRole, textLabel3);

        commentEdit = new KLineEdit(KPropertiesDesktopBase);
        commentEdit->setObjectName(QString::fromUtf8("commentEdit"));

        formLayout->setWidget(2, QFormLayout::FieldRole, commentEdit);

        textLabel4 = new QLabel(KPropertiesDesktopBase);
        textLabel4->setObjectName(QString::fromUtf8("textLabel4"));

        formLayout->setWidget(3, QFormLayout::LabelRole, textLabel4);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName(QString::fromUtf8("horizontalLayout"));
        commandEdit = new KLineEdit(KPropertiesDesktopBase);
        commandEdit->setObjectName(QString::fromUtf8("commandEdit"));

        horizontalLayout->addWidget(commandEdit);

        browseButton = new QPushButton(KPropertiesDesktopBase);
        browseButton->setObjectName(QString::fromUtf8("browseButton"));

        horizontalLayout->addWidget(browseButton);


        formLayout->setLayout(3, QFormLayout::FieldRole, horizontalLayout);

        textLabel5 = new QLabel(KPropertiesDesktopBase);
        textLabel5->setObjectName(QString::fromUtf8("textLabel5"));

        formLayout->setWidget(4, QFormLayout::LabelRole, textLabel5);

        pathEdit = new KUrlRequester(KPropertiesDesktopBase);
        pathEdit->setObjectName(QString::fromUtf8("pathEdit"));

        formLayout->setWidget(4, QFormLayout::FieldRole, pathEdit);


        verticalLayout->addLayout(formLayout);

        kseparator = new KSeparator(KPropertiesDesktopBase);
        kseparator->setObjectName(QString::fromUtf8("kseparator"));

        verticalLayout->addWidget(kseparator);

        textLabel7 = new QLabel(KPropertiesDesktopBase);
        textLabel7->setObjectName(QString::fromUtf8("textLabel7"));

        verticalLayout->addWidget(textLabel7);

        filetypeList = new QTreeWidget(KPropertiesDesktopBase);
        filetypeList->setObjectName(QString::fromUtf8("filetypeList"));
        QSizePolicy sizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(1);
        sizePolicy.setHeightForWidth(filetypeList->sizePolicy().hasHeightForWidth());
        filetypeList->setSizePolicy(sizePolicy);
        filetypeList->setRootIsDecorated(false);
        filetypeList->setSortingEnabled(true);
        filetypeList->setAllColumnsShowFocus(true);

        verticalLayout->addWidget(filetypeList);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName(QString::fromUtf8("horizontalLayout_2"));
        addFiletypeButton = new QPushButton(KPropertiesDesktopBase);
        addFiletypeButton->setObjectName(QString::fromUtf8("addFiletypeButton"));

        horizontalLayout_2->addWidget(addFiletypeButton);

        delFiletypeButton = new QPushButton(KPropertiesDesktopBase);
        delFiletypeButton->setObjectName(QString::fromUtf8("delFiletypeButton"));

        horizontalLayout_2->addWidget(delFiletypeButton);

        spacerItem = new QSpacerItem(70, 20, QSizePolicy::MinimumExpanding, QSizePolicy::Minimum);

        horizontalLayout_2->addItem(spacerItem);

        advancedButton = new QPushButton(KPropertiesDesktopBase);
        advancedButton->setObjectName(QString::fromUtf8("advancedButton"));

        horizontalLayout_2->addWidget(advancedButton);


        verticalLayout->addLayout(horizontalLayout_2);

#ifndef UI_QT_NO_SHORTCUT
        nameLabel->setBuddy(nameEdit);
        textLabel2->setBuddy(genNameEdit);
        textLabel3->setBuddy(commentEdit);
        textLabel4->setBuddy(commandEdit);
        textLabel5->setBuddy(pathEdit);
        textLabel7->setBuddy(filetypeList);
#endif // QT_NO_SHORTCUT

        retranslateUi(KPropertiesDesktopBase);

        QMetaObject::connectSlotsByName(KPropertiesDesktopBase);
    } // setupUi

    void retranslateUi(QWidget *KPropertiesDesktopBase)
    {
#ifndef UI_QT_NO_WHATSTHIS
        nameLabel->setWhatsThis(tr2i18n("Type the name you want to give to this application here. This application will appear under this name in the applications menu and in the panel.", 0));
#endif // QT_NO_WHATSTHIS
        nameLabel->setText(tr2i18n("&Name:", 0));
#ifndef UI_QT_NO_WHATSTHIS
        nameEdit->setWhatsThis(tr2i18n("Type the name you want to give to this application here. This application will appear under this name in the applications menu and in the panel.", 0));
#endif // QT_NO_WHATSTHIS
#ifndef UI_QT_NO_WHATSTHIS
        textLabel2->setWhatsThis(tr2i18n("Type the description of this application, based on its use, here. Examples: a dial up application (KPPP) would be \"Dial up tool\".", 0));
#endif // QT_NO_WHATSTHIS
        textLabel2->setText(tr2i18n("&Description:", 0));
#ifndef UI_QT_NO_WHATSTHIS
        genNameEdit->setWhatsThis(tr2i18n("Type the description of this application, based on its use, here. Examples: a dial up application (KPPP) would be \"Dial up tool\".", 0));
#endif // QT_NO_WHATSTHIS
#ifndef UI_QT_NO_WHATSTHIS
        textLabel3->setWhatsThis(tr2i18n("Type any comment you think is useful here.", 0));
#endif // QT_NO_WHATSTHIS
        textLabel3->setText(tr2i18n("Comm&ent:", 0));
#ifndef UI_QT_NO_WHATSTHIS
        commentEdit->setWhatsThis(tr2i18n("Type any comment you think is useful here.", 0));
#endif // QT_NO_WHATSTHIS
#ifndef UI_QT_NO_WHATSTHIS
        textLabel4->setWhatsThis(tr2i18n("Type the command to start this application here.\n"
"\n"
"Following the command, you can have several place holders which will be replaced with the actual values when the actual program is run:\n"
"%f - a single file name\n"
"%F - a list of files; use for applications that can open several local files at once\n"
"%u - a single URL\n"
"%U - a list of URLs\n"
"%d - the directory of the file to open\n"
"%D - a list of directories\n"
"%i - the icon\n"
"%m - the mini-icon\n"
"%c - the caption", 0));
#endif // QT_NO_WHATSTHIS
        textLabel4->setText(tr2i18n("Co&mmand:", 0));
#ifndef UI_QT_NO_WHATSTHIS
        commandEdit->setWhatsThis(tr2i18n("Type the command to start this application here.\n"
"\n"
"Following the command, you can have several place holders which will be replaced with the actual values when the actual program is run:\n"
"%f - a single file name\n"
"%F - a list of files; use for applications that can open several local files at once\n"
"%u - a single URL\n"
"%U - a list of URLs\n"
"%d - the directory of the file to open\n"
"%D - a list of directories\n"
"%i - the icon\n"
"%m - the mini-icon\n"
"%c - the caption", 0));
#endif // QT_NO_WHATSTHIS
#ifndef UI_QT_NO_WHATSTHIS
        browseButton->setWhatsThis(tr2i18n("Click here to browse your file system in order to find the desired executable.", 0));
#endif // QT_NO_WHATSTHIS
        browseButton->setText(tr2i18n("&Browse...", 0));
#ifndef UI_QT_NO_WHATSTHIS
        textLabel5->setWhatsThis(tr2i18n("Sets the working directory for your application.", 0));
#endif // QT_NO_WHATSTHIS
        textLabel5->setText(tr2i18n("&Work path:", 0));
#ifndef UI_QT_NO_WHATSTHIS
        pathEdit->setWhatsThis(tr2i18n("Sets the working directory for your application.", 0));
#endif // QT_NO_WHATSTHIS
#ifndef UI_QT_NO_WHATSTHIS
        textLabel7->setWhatsThis(tr2i18n("<qt><p>This list should show the types of file that your application can handle. This list is organized by <u>mimetypes</u>.</p>\n"
"<p>MIME, Multipurpose Internet (e)Mail Extension, is a standard protocol for identifying the type of data based on filename extensions and correspondent <u>mimetypes</u>. Example: the \"bmp\" part that comes after the dot in flower.bmp indicates that it is a specific kind of image, <u>image/x-bmp</u>. To know which application should open each type of file, the system should be informed about the abilities of each application to handle these extensions and mimetypes.</p>\n"
"<p>If you want to associate this application with one or more mimetypes that are not in this list, click on the button <b>Add</b> below. If there are one or more filetypes that this application cannot handle, you may want to remove them from the list clicking on the button <b>Remove</b> below.</p></qt>", 0));
#endif // QT_NO_WHATSTHIS
        textLabel7->setText(tr2i18n("&Supported file types:", 0));
        QTreeWidgetItem *___qtreewidgetitem = filetypeList->headerItem();
        ___qtreewidgetitem->setText(1, tr2i18n("Description", 0));
        ___qtreewidgetitem->setText(0, tr2i18n("Mimetype", 0));
#ifndef UI_QT_NO_WHATSTHIS
        filetypeList->setWhatsThis(tr2i18n("<qt><p>This list should show the types of file that your application can handle. This list is organized by <u>mimetypes</u>.</p>\n"
"<p>MIME, Multipurpose Internet (e)Mail Extension, is a standard protocol for identifying the type of data based on filename extensions and correspondent <u>mimetypes</u>. Example: the \"bmp\" part that comes after the dot in flower.bmp indicates that it is a specific kind of image, <u>image/x-bmp</u>. To know which application should open each type of file, the system should be informed about the abilities of each application to handle these extensions and mimetypes.</p>\n"
"<p>If you want to associate this application with one or more mimetypes that are not in this list, click on the button <b>Add</b> below. If there are one or more filetypes that this application cannot handle, you may want to remove them from the list clicking on the button <b>Remove</b> below.</p></qt>", 0));
#endif // QT_NO_WHATSTHIS
#ifndef UI_QT_NO_WHATSTHIS
        addFiletypeButton->setWhatsThis(tr2i18n("Click on this button if you want to add a type of file (mimetype) that your application can handle.", 0));
#endif // QT_NO_WHATSTHIS
        addFiletypeButton->setText(tr2i18n("Add...", 0));
#ifndef UI_QT_NO_WHATSTHIS
        delFiletypeButton->setWhatsThis(tr2i18n("If you want to remove a type of file (mimetype) that your application cannot handle, select the mimetype in the list above and click on this button.", 0));
#endif // QT_NO_WHATSTHIS
        delFiletypeButton->setText(tr2i18n("Remove", 0));
#ifndef UI_QT_NO_WHATSTHIS
        advancedButton->setWhatsThis(tr2i18n("Click here to modify the way this application will run, launch feedback, D-Bus options or to run it as a different user.", 0));
#endif // QT_NO_WHATSTHIS
        advancedButton->setText(tr2i18n("Ad&vanced Options", 0));
        Q_UNUSED(KPropertiesDesktopBase);
    } // retranslateUi

};

namespace Ui {
    class KPropertiesDesktopBase: public Ui_KPropertiesDesktopBase {};
} // namespace Ui

QT_END_NAMESPACE

#endif // KPROPERTIESDESKTOPBASE_H

