/****************************************************************************
** Meta object code from reading C++ file 'itemsviewbasedelegate.h'
**
** Created by: The Qt Meta Object Compiler version 63 (Qt 4.8.7)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../src/kdelibs-4.14.38/knewstuff/knewstuff3/ui/itemsviewbasedelegate.h"
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'itemsviewbasedelegate.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 63
#error "This file was generated using the moc from 4.8.7. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
static const uint qt_meta_data_KNS3__ItemsViewBaseDelegate[] = {

 // content:
       6,       // revision
       0,       // classname
       0,    0, // classinfo
       7,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: signature, parameters, type, tag, flags
      35,   29,   28,   28, 0x05,

 // slots: signature, parameters, type, tag, flags
      93,   79,   74,   28, 0x09,
     123,   28,   28,   28, 0x09,
     151,  144,   28,   28, 0x09,
     192,  188,   28,   28, 0x09,
     223,  217,   28,   28, 0x09,
     255,   28,   28,   28, 0x09,

       0        // eod
};

static const char qt_meta_stringdata_KNS3__ItemsViewBaseDelegate[] = {
    "KNS3::ItemsViewBaseDelegate\0\0entry\0"
    "signalShowDetails(KNS3::EntryInternal)\0"
    "bool\0watched,event\0eventFilter(QObject*,QEvent*)\0"
    "slotInstallClicked()\0action\0"
    "slotInstallActionTriggered(QAction*)\0"
    "url\0slotLinkClicked(QString)\0index\0"
    "slotDetailsClicked(QModelIndex)\0"
    "slotDetailsClicked()\0"
};

void KNS3::ItemsViewBaseDelegate::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        Q_ASSERT(staticMetaObject.cast(_o));
        ItemsViewBaseDelegate *_t = static_cast<ItemsViewBaseDelegate *>(_o);
        switch (_id) {
        case 0: _t->signalShowDetails((*reinterpret_cast< const KNS3::EntryInternal(*)>(_a[1]))); break;
        case 1: { bool _r = _t->eventFilter((*reinterpret_cast< QObject*(*)>(_a[1])),(*reinterpret_cast< QEvent*(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = _r; }  break;
        case 2: _t->slotInstallClicked(); break;
        case 3: _t->slotInstallActionTriggered((*reinterpret_cast< QAction*(*)>(_a[1]))); break;
        case 4: _t->slotLinkClicked((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 5: _t->slotDetailsClicked((*reinterpret_cast< const QModelIndex(*)>(_a[1]))); break;
        case 6: _t->slotDetailsClicked(); break;
        default: ;
        }
    }
}

const QMetaObjectExtraData KNS3::ItemsViewBaseDelegate::staticMetaObjectExtraData = {
    0,  qt_static_metacall 
};

const QMetaObject KNS3::ItemsViewBaseDelegate::staticMetaObject = {
    { &KWidgetItemDelegate::staticMetaObject, qt_meta_stringdata_KNS3__ItemsViewBaseDelegate,
      qt_meta_data_KNS3__ItemsViewBaseDelegate, &staticMetaObjectExtraData }
};

#ifdef Q_NO_DATA_RELOCATION
const QMetaObject &KNS3::ItemsViewBaseDelegate::getStaticMetaObject() { return staticMetaObject; }
#endif //Q_NO_DATA_RELOCATION

const QMetaObject *KNS3::ItemsViewBaseDelegate::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->metaObject : &staticMetaObject;
}

void *KNS3::ItemsViewBaseDelegate::qt_metacast(const char *_clname)
{
    if (!_clname) return 0;
    if (!strcmp(_clname, qt_meta_stringdata_KNS3__ItemsViewBaseDelegate))
        return static_cast<void*>(const_cast< ItemsViewBaseDelegate*>(this));
    return KWidgetItemDelegate::qt_metacast(_clname);
}

int KNS3::ItemsViewBaseDelegate::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = KWidgetItemDelegate::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    return _id;
}

// SIGNAL 0
void KNS3::ItemsViewBaseDelegate::signalShowDetails(const KNS3::EntryInternal & _t1)
{
    void *_a[] = { 0, const_cast<void*>(reinterpret_cast<const void*>(&_t1)) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}
QT_END_MOC_NAMESPACE
