/****************************************************************************
** Meta object code from reading C++ file 'fstabstorageaccess.h'
**
** Created by: The Qt Meta Object Compiler version 63 (Qt 4.8.7)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../src/kdelibs-4.14.38/solid/solid/backends/fstab/fstabstorageaccess.h"
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'fstabstorageaccess.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 63
#error "This file was generated using the moc from 4.8.7. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
static const uint qt_meta_data_Solid__Backends__Fstab__FstabStorageAccess[] = {

 // content:
       6,       // revision
       0,       // classname
       0,    0, // classinfo
      13,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       5,       // signalCount

 // signals: signature, parameters, type, tag, flags
      59,   44,   43,   43, 0x05,
     109,   94,   43,   43, 0x05,
     154,   94,   43,   43, 0x05,
     206,  202,   43,   43, 0x05,
     230,  202,   43,   43, 0x05,

 // slots: signature, parameters, type, tag, flags
     277,  257,   43,   43, 0x08,
     321,  257,   43,   43, 0x08,
     375,  368,   43,   43, 0x08,
     398,   43,   43,   43, 0x08,
     419,   43,   43,   43, 0x08,
     458,  440,   43,   43, 0x08,
     485,   43,   43,   43, 0x08,
     509,  440,   43,   43, 0x08,

       0        // eod
};

static const char qt_meta_stringdata_Solid__Backends__Fstab__FstabStorageAccess[] = {
    "Solid::Backends::Fstab::FstabStorageAccess\0"
    "\0accessible,udi\0accessibilityChanged(bool,QString)\0"
    "error,data,udi\0"
    "setupDone(Solid::ErrorType,QVariant,QString)\0"
    "teardownDone(Solid::ErrorType,QVariant,QString)\0"
    "udi\0setupRequested(QString)\0"
    "teardownRequested(QString)\0"
    "exitCode,exitStatus\0"
    "slotSetupFinished(int,QProcess::ExitStatus)\0"
    "slotTeardownFinished(int,QProcess::ExitStatus)\0"
    "device\0onMtabChanged(QString)\0"
    "connectDBusSignals()\0slotSetupRequested()\0"
    "error,errorString\0slotSetupDone(int,QString)\0"
    "slotTeardownRequested()\0"
    "slotTeardownDone(int,QString)\0"
};

void Solid::Backends::Fstab::FstabStorageAccess::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        Q_ASSERT(staticMetaObject.cast(_o));
        FstabStorageAccess *_t = static_cast<FstabStorageAccess *>(_o);
        switch (_id) {
        case 0: _t->accessibilityChanged((*reinterpret_cast< bool(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 1: _t->setupDone((*reinterpret_cast< Solid::ErrorType(*)>(_a[1])),(*reinterpret_cast< QVariant(*)>(_a[2])),(*reinterpret_cast< const QString(*)>(_a[3]))); break;
        case 2: _t->teardownDone((*reinterpret_cast< Solid::ErrorType(*)>(_a[1])),(*reinterpret_cast< QVariant(*)>(_a[2])),(*reinterpret_cast< const QString(*)>(_a[3]))); break;
        case 3: _t->setupRequested((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 4: _t->teardownRequested((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 5: _t->slotSetupFinished((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< QProcess::ExitStatus(*)>(_a[2]))); break;
        case 6: _t->slotTeardownFinished((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< QProcess::ExitStatus(*)>(_a[2]))); break;
        case 7: _t->onMtabChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 8: _t->connectDBusSignals(); break;
        case 9: _t->slotSetupRequested(); break;
        case 10: _t->slotSetupDone((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 11: _t->slotTeardownRequested(); break;
        case 12: _t->slotTeardownDone((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        default: ;
        }
    }
}

const QMetaObjectExtraData Solid::Backends::Fstab::FstabStorageAccess::staticMetaObjectExtraData = {
    0,  qt_static_metacall 
};

const QMetaObject Solid::Backends::Fstab::FstabStorageAccess::staticMetaObject = {
    { &QObject::staticMetaObject, qt_meta_stringdata_Solid__Backends__Fstab__FstabStorageAccess,
      qt_meta_data_Solid__Backends__Fstab__FstabStorageAccess, &staticMetaObjectExtraData }
};

#ifdef Q_NO_DATA_RELOCATION
const QMetaObject &Solid::Backends::Fstab::FstabStorageAccess::getStaticMetaObject() { return staticMetaObject; }
#endif //Q_NO_DATA_RELOCATION

const QMetaObject *Solid::Backends::Fstab::FstabStorageAccess::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->metaObject : &staticMetaObject;
}

void *Solid::Backends::Fstab::FstabStorageAccess::qt_metacast(const char *_clname)
{
    if (!_clname) return 0;
    if (!strcmp(_clname, qt_meta_stringdata_Solid__Backends__Fstab__FstabStorageAccess))
        return static_cast<void*>(const_cast< FstabStorageAccess*>(this));
    if (!strcmp(_clname, "Solid::Ifaces::StorageAccess"))
        return static_cast< Solid::Ifaces::StorageAccess*>(const_cast< FstabStorageAccess*>(this));
    if (!strcmp(_clname, "org.kde.Solid.Ifaces.StorageAccess/0.1"))
        return static_cast< Solid::Ifaces::StorageAccess*>(const_cast< FstabStorageAccess*>(this));
    return QObject::qt_metacast(_clname);
}

int Solid::Backends::Fstab::FstabStorageAccess::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 13)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    }
    return _id;
}

// SIGNAL 0
void Solid::Backends::Fstab::FstabStorageAccess::accessibilityChanged(bool _t1, const QString & _t2)
{
    void *_a[] = { 0, const_cast<void*>(reinterpret_cast<const void*>(&_t1)), const_cast<void*>(reinterpret_cast<const void*>(&_t2)) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void Solid::Backends::Fstab::FstabStorageAccess::setupDone(Solid::ErrorType _t1, QVariant _t2, const QString & _t3)
{
    void *_a[] = { 0, const_cast<void*>(reinterpret_cast<const void*>(&_t1)), const_cast<void*>(reinterpret_cast<const void*>(&_t2)), const_cast<void*>(reinterpret_cast<const void*>(&_t3)) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void Solid::Backends::Fstab::FstabStorageAccess::teardownDone(Solid::ErrorType _t1, QVariant _t2, const QString & _t3)
{
    void *_a[] = { 0, const_cast<void*>(reinterpret_cast<const void*>(&_t1)), const_cast<void*>(reinterpret_cast<const void*>(&_t2)), const_cast<void*>(reinterpret_cast<const void*>(&_t3)) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void Solid::Backends::Fstab::FstabStorageAccess::setupRequested(const QString & _t1)
{
    void *_a[] = { 0, const_cast<void*>(reinterpret_cast<const void*>(&_t1)) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void Solid::Backends::Fstab::FstabStorageAccess::teardownRequested(const QString & _t1)
{
    void *_a[] = { 0, const_cast<void*>(reinterpret_cast<const void*>(&_t1)) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}
QT_END_MOC_NAMESPACE
